#=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~=~
#                                      numgeo - GiD interface
#                       Copyright (C) 2023-today Jan Machacek, Rahul Dogra
#=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~=~
#
# History
# - 26.11.2023, J. Machacek - Initial version
#
#
#=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~=~

proc numgeo::return_eltype {dim formulation integration shape numnodes} {
    # 2D (plane-strain)
    if {$dim == "2D (plane-strain)"} {
        # Triangular elements
        if {$shape == "Triangular"} {
            if {$numnodes == "3"} {
                if {$formulation == "Single-phase solid material"} {
                    return "u3-solid"
                } elseif {$formulation == "Single-phase porous material"} {
                    return "u3"
                }
            } elseif {$numnodes == "6"} {
                if {$integration == "full"} {
                    set red ""
                } else {
                    set red "-red"
                }
                if {$formulation == "Single-phase solid material"} {
                    return "u6-solid"
                } elseif {$formulation == "Single-phase porous material"} {
                    return "u6"
                } elseif {$formulation == "Two-phase u-p material"} {
                    return "u6p3-sat"
                } elseif {$formulation == "Two-phase u-p-U material"} {
                    return "u6p3u6-sat"
                }
            }
        }
        # Quadrilateral elements
        if {$shape == "Quadrilateral"} {
            if {$numnodes == "4"} {
                if {$integration == "full"} {
                    set red ""
                } else {
                    set red "-red"
                }
                if {$formulation == "Single-phase solid material"} {
                    return "u4-solid$red"
                } elseif {$formulation == "Single-phase porous material"} {
                    return "u4$red"
                } elseif {$formulation == "Two-phase u-U material"} {
                    return "u4u4$red"
                } elseif {$formulation == "Two-phase u-p material"} {
                    return "u4p4-sat-stab$red"
                }
            } elseif {$numnodes == "8"} {
                if {$integration == "full"} {
                    set red ""
                } else {
                    set red "-red"
                }
                if {$formulation == "Single-phase solid material"} {
                    return "u8-solid$red"
                } elseif {$formulation == "Single-phase porous material"} {
                    return "u8$red"
                } elseif {$formulation == "Two-phase u-p material"} {
                    return "u8p4-sat$red"
                } elseif {$formulation == "Two-phase u-p-U material"} {
                    return "u8p4u8-sat$red"
                } elseif {$formulation == "Two-phase u-U material"} {
                    return "u8u8$red"
                } 
            }
        }
        # Additional cases for triangular elements
        if {$shape == "Triangular"} {
            if {$numnodes == "6"} {
                if {$integration == "full" || $integration == "reduced"} {
                    if {$formulation == "Three-phase porous material (simplified)"} {
                        return "u6p3"
                    } elseif {$formulation == "Three-phase porous material (complete)"} {
                        return "u6pp3"
                    }
                }
            } elseif {$numnodes == "3"} {
                if {$integration == "full" || $integration == "reduced"} {
                    if {$formulation == "Three-phase porous material (simplified)"} {
                        return "u3p3-stab"
                    } elseif {$formulation == "Three-phase porous material (complete)"} {
                        return "u3p3-stab"
                    }
                }
            }
        }
        # Additional cases for quadrilateral elements

        if {$shape == "Quadrilateral"} {
            if {$numnodes == "4"} {
                if {$integration == "full" || $integration == "reduced"} {
                    if {$formulation == "Three-phase porous material (simplified)"} {
                        return "u4p4-stab-red"
                    } elseif {$formulation == "Three-phase porous material (complete)"} {
                        return "u4p4-stab-red"
                    }
                }
            }  elseif {$numnodes == "8"} {
                if {$integration == "full" || $integration == "reduced"} {
                    if {$formulation == "Three-phase porous material (simplified)"} {
                        return "u8pp4"
                    } elseif {$formulation == "Three-phase porous material (complete)"} {
                        return "u8p4"
                    }
                }
            }
        }
    } elseif {$dim == "2D: Axissymmetric"} {
        # Additional case for 2D axissymmetric
        if {$shape == "Triangular"} {
            if {$numnodes == "6"} {
                if {$integration == "full" || $integration == "reduced"} {
                    if {$formulation == "Three-phase porous material (simplified)"} {
                        return "u6p3-ax"
                    } elseif {$formulation == "Three-phase porous material (complete)"} {
                        return "u6pp3-ax"
                    }
                }
            } elseif {$numnodes == "3"} {
                if {$integration == "full" || $integration == "reduced"} {
                    if {$formulation == "Three-phase porous material (simplified)"} {
                        return "u3p3-stab-ax"
                    } elseif {$formulation == "Three-phase porous material (complete)"} {
                        return "u3p3-stab-ax"
                    }
                }
            }
        }
		if {$shape == "Quadrilateral"} {
            if {$numnodes == "4"} {
                if {$integration == "full" || $integration == "reduced"} {
                    if {$formulation == "Three-phase porous material (simplified)"} {
                        return "u4p4-stab-ax-red"
                    } elseif {$formulation == "Three-phase porous material (complete)"} {
                        return "u4p4-stab-ax-red"
                    }
                }
            }  elseif {$numnodes == "8"} {
                if {$integration == "full" || $integration == "reduced"} {
                    if {$formulation == "Three-phase porous material (simplified)"} {
                        return "u8p4-ax"
                    } elseif {$formulation == "Three-phase porous material (complete)"} {
                        return "u8pp4-ax"
                    }
                }
            }
        }
		if {$shape == "Triangular"} {
            if {$numnodes == "6"} {
                if {$formulation == "Single-phase solid material"} {
                    if {$integration == "full"} {
                        return "u6-solid-ax"
                    } elseif {$integration == "reduced"} {
						return "u6-solid-ax"
					}
                }  elseif {$formulation == "Single-phase porous material"} {
                    if {$integration == "full"} {
                        return "u6-ax"
                    } elseif {$integration == "reduced"} {
						return "u6-ax"
					}
                }  elseif {$formulation == "Two-phase u-p material"} {
                    if {$integration == "full"} {
                        return "u6p3-sat-ax"
                    } elseif {$integration == "reduced"} {
						return "u6p3-sat-ax"
					}
				}  elseif {$formulation == "Two-phase u-p-U material"} {
                    if {$integration == "full"} {
                        return "u6p3u6-sat-ax"
                    } elseif {$integration == "reduced"} {
						return "u6p3u6-sat-ax"
					}
				}
            }
        }  elseif {$shape == "Quadrilateral"} {
            if {$numnodes == "4"} {
                if {$formulation == "Single-phase solid material"} {
                    if {$integration == "full"} {
                        return "u4-solid-ax"
                    } elseif {$integration == "reduced"} {
						return "u4-solid-ax-red"
					}
                }  elseif {$formulation == "Single-phase porous material"} {
                    if {$integration == "full"} {
                        return "u4-ax"
                    } elseif {$integration == "reduced"} {
						return "u4-ax"
					}
                }  elseif {$formulation == "Two-phase u-U material"} {
                    if {$integration == "full"} {
                        return "u4u4-ax"
                    } elseif {$integration == "reduced"} {
						return "u4u4-ax-red"
					}
                }  elseif {$formulation == "Two-phase u-p material"} {
                    if {$integration == "full"} {
                        return "u4u4-sat-ax"
                    } elseif {$integration == "reduced"} {
						return "u4u4-sat-ax"
					}
                }
            } elseif {$numnodes == "8"} {
                if {$formulation == "Single-phase solid material"} {
                    if {$integration == "full"} {
                        return "u8-solid-ax"
                    } elseif {$integration == "reduced"} {
						return "u8-solid-ax"
					}
                }  elseif {$formulation == "Single-phase porous material"} {
                    if {$integration == "full"} {
                        return "u8-ax"
                    } elseif {$integration == "reduced"} {
						return "u8-ax"
					}
                }  elseif {$formulation == "Two-phase u-p material"} {
                    if {$integration == "full"} {
                        return "u8p4-sat-ax"
                    } elseif {$integration == "reduced"} {
						return "u8p4-sat-ax-red"
					}
				}  elseif {$formulation == "Two-phase u-p-U material"} {
                    if {$integration == "full"} {
                        return "u8p4u8-sat-ax"
                    } elseif {$integration == "reduced"} {
						return "u8p4u8-sat-ax-red"
					}
				}  elseif {$formulation == "Two-phase u-U material"} {
                    if {$integration == "full"} {
                        return "u8u8-ax"
                    } elseif {$integration == "reduced"} {
						return "u8u8-ax-red"
					}
				}
            }
        }
    }  elseif {$dim == "3D"} {
		if {$shape == "Hexahedra"} {
            if {$numnodes == "8"} {
                if {$formulation == "Single-phase solid material"} {
                    if {$integration == "full"} {
                        return "u8-solid-3D"
                    } elseif {$integration == "reduced"} {
						return "u8-solid-3D-red"
					}
                }  elseif {$formulation == "Single-phase porous material"} {
                    if {$integration == "full"} {
                        return "u8-3D"
                    } elseif {$integration == "reduced"} {
						return "u8-3D"
					}
                }  elseif {$formulation == "Two-phase u-U material"} {
                    if {$integration == "full"} {
                        return "u8u8-3D"
                    } elseif {$integration == "reduced"} {
						return "u8u8-3D-red"
					}
				}  
        } elseif {$numnodes == "20"} {
                if {$formulation == "Single-phase solid material"} {
                    if {$integration == "full"} {
                        return "u20-solid"
                    } elseif {$integration == "reduced"} {
						return "u20-solid-red"
					}
                }  elseif {$formulation == "Single-phase porous material"} {
                    if {$integration == "full"} {
                        return "u20"
                    } elseif {$integration == "reduced"} {
						return "u20-red"
					}
                }  elseif {$formulation == "Two-phase u-U material"} {
                    if {$integration == "full"} {
                        return "u20u20"
                    } elseif {$integration == "reduced"} {
						return "u20u20"
					}
                }  elseif {$formulation == "Two-phase u-p material"} {
                    if {$integration == "full"} {
                        return "u20p8-sat"
                    } elseif {$integration == "reduced"} {
						return "u20p8-sat-red"
					}
                }  elseif {$formulation == "Two-phase u-p-U material"} {
                    if {$integration == "full"} {
                        return "u20p8u20-sat"
                    } elseif {$integration == "reduced"} {
						return "u20p8u20-sat-red"
					}
                }
			}
		}
    }
}