#=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~=~
#                                      numgeo - GiD interface
#                       Copyright (C) 2023-today Jan Machacek, Rahul Dogra
#=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~=~
#
# History
# - 26.11.2023, J. Machacek - Initial version
#
# Memo:
# - get_domnode_attribute returns the current value of "n" (name) of $group
#
#
#=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~~=~=~

proc numgeo::WriteCalculationFile { filename ModelName} {

    set filenameMesh $ModelName
	append filenameMesh "-mesh"
    append filename "-calculation.inp"

    customlib::InitWriteFile $filename
	
    # xml document to get some tree data
    set root [$::gid_groups_conds::doc documentElement] ;
	
    customlib::WriteString "**=================================================================="
    customlib::WriteString "**                        numgeo input file"    
    customlib::WriteString "**=================================================================="
    customlib::WriteString "**"    
    customlib::WriteString "** Input file for the finite element code numgeo (Machacek & Staubach)"
    customlib::WriteString "**"   
    customlib::WriteString "** The input file was created by numgeo-GiD (Machacek & Dogra)"   
    customlib::WriteString "**"   
    customlib::WriteString "** numgeo is available from www.numgeo.de"   
    customlib::WriteString "** numgeo-GiD can be downloaded from github XXX" 
    customlib::WriteString "**"  
    customlib::WriteString "** Contact:"
    customlib::WriteString "** - Jan Machacek, jan.machacek@tu-darmstadt.de"
    customlib::WriteString "** - Rahul Dogra, rahul.dogra@tu-darmstadt.de"
    customlib::WriteString "**"
    customlib::WriteString "**=================================================================="
	
    #
	# Import mesh
	# 
    customlib::WriteString ""
	customlib::WriteString "**"
	customlib::WriteString "** Mesh"
	customlib::WriteString "**"
	customlib::WriteString ""
	GiD_WriteCalculationFile puts "*Include = $filenameMesh"
	
	
    #
	# Solid Section
	# 
    customlib::WriteString ""
	customlib::WriteString "**"
	customlib::WriteString "** Solid Sections"
	customlib::WriteString "**"
	customlib::WriteString ""
	set document [$::gid_groups_conds::doc documentElement]
	set xp [format_xpath {condition[@n="_solid_sections_"]/group}]
	foreach group [$document selectNodes $xp] {
	  set group_name [get_domnode_attribute $group n]
	  set node [$group selectNodes "./value\[@n = '_material_'\]"]
	  set material [get_domnode_attribute $node v]
	  GiD_WriteCalculationFile puts "*Solid section, elset=$group_name, material=$material"
	}

    #
	# Materials
	# 
    customlib::WriteString ""
	customlib::WriteString "**"
	customlib::WriteString "** Materials"
	customlib::WriteString "**"
	
	# create a list of material definitions
	set xp [format_xpath {container[@n="_materials_"]/blockdata}]
	set list [$root selectNodes $xp]
	
	# loop over all material definitions
	foreach imat $list {
	
	  customlib::WriteString ""
	
	  # material name and number of phases
	  set matname [$imat getAttribute "name"]
	  set node [$imat selectNodes [format_xpath {value[@n="_nphases_"]}]]
	  set nphases [$node getAttribute "v"]
	  GiD_WriteCalculationFile puts "*Material, name=$matname, phases=$nphases"
	    
	  # mechanical model (stess-strain)
	  set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_mechanical_model_"]}]]
	  set model_name [$node getAttribute "v"]
	  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA_m_"]}]]
	  set hca [$node getAttribute "v"]
	  # Linear Elasticity
	  if {$model_name == "Linear Elasticity"} {
	    set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_LE0_"]}]]
	    set name [$node getAttribute "v"]
	    set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_LE1_"]}]]
	    set p1 [$node getAttribute "v"]
	    set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_LE2_"]}]]
	    set p2 [$node getAttribute "v"]
	    if {$hca == "No"} {
		  GiD_WriteCalculationFile puts "*Mechanical = $name"
		  GiD_WriteCalculationFile puts "$p1, $p2"
		} elseif {$hca == "Yes"} {
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA0_"]}]]
		  set name [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA1_"]}]]
		  set HCA1 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA2_"]}]]
		  set HCA2 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA3_"]}]]
		  set HCA3 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA4_"]}]]
		  set HCA4 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA5_"]}]]
		  set HCA5 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA6_"]}]]
		  set HCA6 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA7_"]}]]
		  set HCA7 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA8_"]}]]
		  set HCA8 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA9_"]}]]
		  set HCA9 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA10_"]}]]
		  set HCA10 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA11_"]}]]
		  set HCA11 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA12_"]}]]
		  set HCA12 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA13_"]}]]
		  set HCA13 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA14_"]}]]
		  set HCA14 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA15_"]}]]
		  set HCA15 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA16_"]}]]
		  set HCA16 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA17_"]}]]
		  set HCA17 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA18_"]}]]
		  set HCA18 [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "*Mechanical = HCA_Linear_elasticity"
		  GiD_WriteCalculationFile puts "$p1, $p2"
		  GiD_WriteCalculationFile puts "$HCA1, $HCA2, $HCA3, $HCA4, $HCA5, 0, $HCA6, 0"
		  GiD_WriteCalculationFile puts "$HCA7, $HCA8, $HCA9, $HCA10, $HCA11, $HCA12, $HCA13, $HCA14"
		  GiD_WriteCalculationFile puts "$HCA15, $HCA16, $HCA17, $HCA18"
		}  
	  # Mohr-Coulomb
	  } elseif {$model_name == "Mohr-Coulomb"} {
	  	set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MC0_"]}]]
	    set name [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MC1_"]}]]
	    set p1 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MC2_"]}]]
	    set p2 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MC3_"]}]]
	    set p3 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MC4_"]}]]
	    set p4 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MC5_"]}]]
	    set p5 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MC6_"]}]]
	    set p6 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Mechanical = $name"
		GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4, $p5, $p6"
	  } elseif {$model_name == "Hypoplastic+IGS"} {
	  	set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP0_"]}]]
	    set name [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP1_"]}]]
	    set p1 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP2_"]}]]
	    set p2 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP3_"]}]]
	    set p3 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP4_"]}]]
	    set p4 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP5_"]}]]
	    set p5 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP6_"]}]]
	    set p6 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP7_"]}]]
	    set p7 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP8_"]}]]
	    set p8 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP9_"]}]]
	    set p9 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP10_"]}]]
	    set p10 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP11_"]}]]
	    set p11 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP12_"]}]]
	    set p12 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP13_"]}]]
	    set p13 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP14_"]}]]
	    set p14 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HP15_"]}]]
	    set p15 [$node getAttribute "v"]
		if {$hca == "No"} {
		GiD_WriteCalculationFile puts "*Mechanical = $name"
		GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4, $p5, $p6, $p7, $p8"
		GiD_WriteCalculationFile puts "$p9, $p10, $p11, $p12, $p13, $p14, $p15"
		} elseif {$hca == "Yes"} {
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA0_"]}]]
		  set name [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA1_"]}]]
		  set HCA1 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA2_"]}]]
		  set HCA2 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA3_"]}]]
		  set HCA3 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA4_"]}]]
		  set HCA4 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA5_"]}]]
		  set HCA5 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA6_"]}]]
		  set HCA6 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA7_"]}]]
		  set HCA7 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA8_"]}]]
		  set HCA8 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA9_"]}]]
		  set HCA9 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA10_"]}]]
		  set HCA10 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA11_"]}]]
		  set HCA11 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA12_"]}]]
		  set HCA12 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA13_"]}]]
		  set HCA13 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA14_"]}]]
		  set HCA14 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA15_"]}]]
		  set HCA15 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA16_"]}]]
		  set HCA16 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA17_"]}]]
		  set HCA17 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA18_"]}]]
		  set HCA18 [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "*Mechanical = HCA_Hypoplasticity"
		  GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4, $p5, $p6, $p7, $p8"
		  GiD_WriteCalculationFile puts "$p9, $p10, $p11, $p12, $p13, $p14, $p15"
		  GiD_WriteCalculationFile puts "$HCA1, $HCA2, $HCA3, $HCA4, $HCA5, 0, $HCA6, 0"
		  GiD_WriteCalculationFile puts "$HCA7, $HCA8, $HCA9, $HCA10, $HCA11, $HCA12, $HCA13, $HCA14"
		  GiD_WriteCalculationFile puts "$HCA15, $HCA16, $HCA17, $HCA18"
		}  
	  } elseif {$model_name == "Hypo-ISA"} {
	  	set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI0_"]}]]
	    set name [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI1_"]}]]
	    set p1 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI2_"]}]]
	    set p2 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI3_"]}]]
	    set p3 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI4_"]}]]
	    set p4 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI5_"]}]]
	    set p5 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI6_"]}]]
	    set p6 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI7_"]}]]
	    set p7 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI8_"]}]]
	    set p8 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI9_"]}]]
	    set p9 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI10_"]}]]
	    set p10 [$node getAttribute "v"]                                                     
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI11_"]}]]
	    set p11 [$node getAttribute "v"]                                                     
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI12_"]}]]
	    set p12 [$node getAttribute "v"]                                                     
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI13_"]}]]
	    set p13 [$node getAttribute "v"]                                                     
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI14_"]}]]
	    set p14 [$node getAttribute "v"]                                                     
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI15_"]}]]
	    set p15 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI16_"]}]]
	    set p16 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI17_"]}]]
	    set p17 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI18_"]}]]
	    set p18 [$node getAttribute "v"]                                                      
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI19_"]}]]
	    set p19 [$node getAttribute "v"] 
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI20_"]}]]
	    set p20 [$node getAttribute "v"]	
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_HPI21_"]}]]
	    set p21 [$node getAttribute "v"]
		if {$hca == "No"} {
		GiD_WriteCalculationFile puts "*Mechanical = $name"
		GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4, $p5, $p6, $p7, $p8"
		GiD_WriteCalculationFile puts "$p9, $p10, $p11, $p12, $p13, $p14, $p15, $p16"
		GiD_WriteCalculationFile puts "$p17, $p18, $p19, $p20, $p21"
		} elseif {$hca == "Yes"} {
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA0_"]}]]
		  set name [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA1_"]}]]
		  set HCA1 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA2_"]}]]
		  set HCA2 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA3_"]}]]
		  set HCA3 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA4_"]}]]
		  set HCA4 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA5_"]}]]
		  set HCA5 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA6_"]}]]
		  set HCA6 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA7_"]}]]
		  set HCA7 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA8_"]}]]
		  set HCA8 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA9_"]}]]
		  set HCA9 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA10_"]}]]
		  set HCA10 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA11_"]}]]
		  set HCA11 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA12_"]}]]
		  set HCA12 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA13_"]}]]
		  set HCA13 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA14_"]}]]
		  set HCA14 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA15_"]}]]
		  set HCA15 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA16_"]}]]
		  set HCA16 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA17_"]}]]
		  set HCA17 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA18_"]}]]
		  set HCA18 [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "*Mechanical = HCA-HYPO-ISA"
		  GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4, $p5, $p6, $p7, $p8"
		  GiD_WriteCalculationFile puts "$p9, $p10, $p11, $p12, $p13, $p14, $p15, $p16"
		  GiD_WriteCalculationFile puts "$p17, $p18, $p19, $p20, $p21"
		  GiD_WriteCalculationFile puts "$HCA1, $HCA2, $HCA3, $HCA4, $HCA5, 0, $HCA6, 0"
		  GiD_WriteCalculationFile puts "$HCA7, $HCA8, $HCA9, $HCA10, $HCA11, $HCA12, $HCA13, $HCA14"
		  GiD_WriteCalculationFile puts "$HCA15, $HCA16, $HCA17, $HCA18"
		}
	  } elseif {$model_name == "Sanisand"} {
	  	set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS0_"]}]]
	    set name [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS1_"]}]]
	    set p1 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS2_"]}]]
	    set p2 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS3_"]}]]
	    set p3 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS4_"]}]]
	    set p4 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS5_"]}]]
	    set p5 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS6_"]}]]
	    set p6 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS7_"]}]]
	    set p7 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS8_"]}]]
	    set p8 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS9_"]}]]
	    set p9 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS10_"]}]]
	    set p10 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS11_"]}]]
	    set p11 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS12_"]}]]
	    set p12 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS13_"]}]]
	    set p13 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS14_"]}]]
	    set p14 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS15_"]}]]
	    set p15 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_SS16_"]}]]
	    set p16 [$node getAttribute "v"]
		if {$hca == "No"} {
		  GiD_WriteCalculationFile puts "*Mechanical = $name-2"
		  GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4, $p5, $p6, $p7, $p8"
		  GiD_WriteCalculationFile puts "$p9, $p10, $p11, $p12, $p13, $p14, $p15, $p16"
		} elseif {$hca == "Yes"} {
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA0_"]}]]
		  set name [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA1_"]}]]
		  set HCA1 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA2_"]}]]
		  set HCA2 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA3_"]}]]
		  set HCA3 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA4_"]}]]
		  set HCA4 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA5_"]}]]
		  set HCA5 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA6_"]}]]
		  set HCA6 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA7_"]}]]
		  set HCA7 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA8_"]}]]
		  set HCA8 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA9_"]}]]
		  set HCA9 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA10_"]}]]
		  set HCA10 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA11_"]}]]
		  set HCA11 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA12_"]}]]
		  set HCA12 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA13_"]}]]
		  set HCA13 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA14_"]}]]
		  set HCA14 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA15_"]}]]
		  set HCA15 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA16_"]}]]
		  set HCA16 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA17_"]}]]
		  set HCA17 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA18_"]}]]
		  set HCA18 [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "*Mechanical = HCA-Sanisand-2"
		  GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4, $p5, $p6, $p7, $p8"
		  GiD_WriteCalculationFile puts "$p9, $p10, $p11, $p12, $p13, $p14, $p15, $p16"
		  GiD_WriteCalculationFile puts "$HCA1, $HCA2, $HCA3, $HCA4, $HCA5, 0, $HCA6, 0"
		  GiD_WriteCalculationFile puts "$HCA7, $HCA8, $HCA9, $HCA10, $HCA11, $HCA12, $HCA13, $HCA14"
		  GiD_WriteCalculationFile puts "$HCA15, $HCA16, $HCA17, $HCA18"
		}
	  } elseif {$model_name == "Modified-Cam-Clay"} {
	  	set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC0_"]}]]
	    set name [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC1_"]}]]
	    set p1 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC2_"]}]]
	    set p2 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC3_"]}]]
	    set p3 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC4_"]}]]
	    set p4 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC5_"]}]]
	    set p5 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC6_"]}]]
	    set p6 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC7_"]}]]
	    set p7 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC8_"]}]]
	    set p8 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC9_"]}]]
	    set p9 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC10_"]}]]
	    set p10 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MCC11_"]}]]
	    set p11 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Mechanical = $name"
		GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4, $p5, $p6, $p7, $p8"
		GiD_WriteCalculationFile puts "$p9, $p10, $p11"
	  } elseif {$model_name == "Barotropic elasticity"} {
	  	set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_BE0_"]}]]
	    set name [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_BE1_"]}]]
	    set p1 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_BE2_"]}]]
	    set p2 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_BE3_"]}]]
	    set p3 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_BE4_"]}]]
	    set p4 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Mechanical = $name"
		GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4"
	  } elseif {$model_name == "Matsuoka-Nakai"} {
	  	set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MN0_"]}]]
	    set name [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MN1_"]}]]
	    set p1 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MN2_"]}]]
	    set p2 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MN3_"]}]]
	    set p3 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MN4_"]}]]
	    set p4 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MN5_"]}]]
	    set p5 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_mechanical_"]/value[@n="_MN6_"]}]]
	    set p6 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Mechanical = $name"
		GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4, $p5, $p6"
	  }
	  # density
	  if {$nphases == "1"} {
	    set node [$imat selectNodes [format_xpath {container[@n="_density_"]/value[@n="phase1"]}]]
	    set density1 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Density"
		GiD_WriteCalculationFile puts $density1
	  } elseif {$nphases == "2"} {
	    set node [$imat selectNodes [format_xpath {container[@n="_density_"]/value[@n="phase1"]}]]
	    set density1 [$node getAttribute "v"]
	    set node [$imat selectNodes [format_xpath {container[@n="_density_"]/value[@n="phase2"]}]]
	    set density2 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Density"
		GiD_WriteCalculationFile puts "$density1, $density2"
	  } elseif {$nphases == "3"} {
	    set node [$imat selectNodes [format_xpath {container[@n="_density_"]/value[@n="phase1"]}]]
	    set density1 [$node getAttribute "v"]
	    set node [$imat selectNodes [format_xpath {container[@n="_density_"]/value[@n="phase2"]}]]
	    set density2 [$node getAttribute "v"]
	    set node [$imat selectNodes [format_xpath {container[@n="_density_"]/value[@n="phase3"]}]]
	    set density3 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Density"
		GiD_WriteCalculationFile puts "$density1, $density2, $density3"
	  }
	  
	  # Bulk modulus
	  if {$nphases == "2"} {
	    set node [$imat selectNodes [format_xpath {container[@n="_bulk_modulus_"]/value[@n="fluid1"]}]]
	    set par1 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Bulk modulus"
		GiD_WriteCalculationFile puts "$par1"
	  } elseif {$nphases == "3"} {
	    set node [$imat selectNodes [format_xpath {container[@n="_bulk_modulus_"]/value[@n="fluid1"]}]]
	    set par1 [$node getAttribute "v"]
	    set node [$imat selectNodes [format_xpath {container[@n="_bulk_modulus_"]/value[@n="fluid2"]}]]
	    set par2 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Bulk modulus"
		GiD_WriteCalculationFile puts "$par1, $par2"
	  }
	  
	  # Dynamic viscosity
	  if {$nphases == "2"} {
	    set node [$imat selectNodes [format_xpath {container[@n="_dynamic_viscosity_"]/value[@n="d_visco1"]}]]
	    set par1 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Dynamic viscosity"
		GiD_WriteCalculationFile puts "$par1"
	  } elseif {$nphases == "3"} {
	    set node [$imat selectNodes [format_xpath {container[@n="_dynamic_viscosity_"]/value[@n="d_visco1"]}]]
	    set par1 [$node getAttribute "v"]
	    set node [$imat selectNodes [format_xpath {container[@n="_dynamic_viscosity_"]/value[@n="d_visco2"]}]]
	    set par2 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Dynamic viscosity"
		GiD_WriteCalculationFile puts "$par1, $par2"
	  }
	  
	  # Permeability
	  if {$nphases == "2"} {
		set node [$imat selectNodes [format_xpath {container[@n="_permeability_"]/value[@n="_PM1_"]}]]
		set par1 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Permeability = $par1"
		set node [$imat selectNodes [format_xpath {container[@n="_permeability_"]/value[@n="_PM2_"]}]]
		set par2 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "$par2"
	  } elseif {$nphases == "3"} {
		set node [$imat selectNodes [format_xpath {container[@n="_permeability_"]/value[@n="_PM1_"]}]]
		set par1 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Permeability = $par1"
		set node [$imat selectNodes [format_xpath {container[@n="_permeability_"]/value[@n="_PM2_"]}]]
		set par2 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "$par2"
	  }

	  
	  # hydraulic model (saturation-suction)
	  if {$nphases == "3"} {
	    set node [$imat selectNodes [format_xpath {container[@n="_hydraulic_"]/value[@n="_hydraulic_model_"]}]]
	    set model_name [$node getAttribute "v"]
		# Van Genuchten
	    if {$model_name == "Van Genuchten"} {
	      set node [$imat selectNodes [format_xpath {container[@n="_hydraulic_"]/value[@n="_VG0_"]}]]
	      set name [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_hydraulic_"]/value[@n="_VG1_"]}]]
	      set p1 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_hydraulic_"]/value[@n="_VG2_"]}]]
	      set p2 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_hydraulic_"]/value[@n="_VG3_"]}]]
	      set p3 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_hydraulic_"]/value[@n="_JA0_"]}]]
	      set par [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "*Hydraulic = $name, swr=$p1"
		  GiD_WriteCalculationFile puts "$p2, $p3"
		  GiD_WriteCalculationFile puts "*Jacobian = $par"
	    }
	  }
	  
	    	  
	  # relative permeability model (relative permeability - saturation)
	  if {$nphases == "3"} {
	    set node [$imat selectNodes [format_xpath {container[@n="_rel_permeability_"]/value[@n="_rel_permeability_model_"]}]]
	    set model_name [$node getAttribute "v"]
		# Van Genuchten
	    if {$model_name == "Van Genuchten"} {
	      set node [$imat selectNodes [format_xpath {container[@n="_rel_permeability_"]/value[@n="_VGR0_"]}]]
	      set name [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_rel_permeability_"]/value[@n="_VGR1_"]}]]
	      set p1 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_rel_permeability_"]/value[@n="_VGR2_"]}]]
	      set p2 [$node getAttribute "v"]
		  set node [$imat selectNodes [format_xpath {container[@n="_rel_permeability_"]/value[@n="_VGR3_"]}]]
	      set p3 [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "*Relative permeability = $name"
		  GiD_WriteCalculationFile puts "$p1, $p2, $p3"
	    }
	  }
	  # Bishop effective stress
	  if {$nphases == "3"} {
	    # Van Genuchten
	    if {$model_name == "Van Genuchten"} {
	      set node [$imat selectNodes [format_xpath {container[@n="_Bishop_"]/value[@n="_BI0_"]}]]
	      set par [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "*Bishop effective stress = $par"
	    }
	  }

	  # Strength reduction
	  set node [$imat selectNodes [format_xpath {container[@n="_strength_reduction_"]/value[@n="_strength_"]}]]
	  set st_re [$node getAttribute "v"]
	    if {$st_re == "Yes"} {
		GiD_WriteCalculationFile puts "*Reducible Strength"
	    } else {
		}
	  
	  # Mimimum pressure
	  set node [$imat selectNodes [format_xpath {container[@n="_stab_par_"]/container[@n="_min_press_"]/value[@n="_min_pre_"]}]]
	  set mi_pr [$node getAttribute "v"]
	    if {$mi_pr == "Yes"} {
		set node [$imat selectNodes [format_xpath {container[@n="_stab_par_"]/container[@n="_min_press_"]/value[@n="_min_p_"]}]]
	    set m_v [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Min pressure"
		GiD_WriteCalculationFile puts "$m_v"
	    } else {
		}
		
	  # Phantom Elasticity
	  set node [$imat selectNodes [format_xpath {container[@n="_stab_par_"]/container[@n="_pham_elas_"]/value[@n="_pham_ela_"]}]]
	  set ph_el [$node getAttribute "v"]
	    if {$ph_el == "Yes"} {
		set node [$imat selectNodes [format_xpath {container[@n="_stab_par_"]/container[@n="_pham_elas_"]/value[@n="_pha_el_1_"]}]]
	    set p_e1 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_stab_par_"]/container[@n="_pham_elas_"]/value[@n="_pha_el_2_"]}]]
	    set p_e2 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Phantom elasticity"
		GiD_WriteCalculationFile puts "$p_e1, $p_e2"
	    } else {
		}	
		
	  # Damping
	  set node [$imat selectNodes [format_xpath {container[@n="_damping_"]/value[@n="_damp_"]}]]
	  set damp [$node getAttribute "v"]
	    if {$damp == "Yes"} {
		set node [$imat selectNodes [format_xpath {container[@n="_damping_"]/value[@n="_damp1_"]}]]
	    set d_a1 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_damping_"]/value[@n="_damp2_"]}]]
	    set d_a2 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*Damping"
		GiD_WriteCalculationFile puts "$d_a1, $d_a2"
	    } else {
		}
		
	  # Hourglass
	    set node [$imat selectNodes [format_xpath {container[@n="_stab_par_"]/container[@n="_hr_glass_"]/value[@n="_hr_gl_"]}]]
	    set hr_gls [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_stab_par_"]/container[@n="_hr_glass_"]/value[@n="_hr_method_"]}]]
	    set hr_1 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_stab_par_"]/container[@n="_hr_glass_"]/value[@n="_hr_m1_"]}]]
	    set hr_2 [$node getAttribute "v"]
		set node [$imat selectNodes [format_xpath {container[@n="_stab_par_"]/container[@n="_hr_glass_"]/value[@n="_hr_m2_"]}]]
	    set hr_3 [$node getAttribute "v"]
		if {$hr_gls == "Yes"} {
			if {$nphases == "1"} {
			  GiD_WriteCalculationFile puts "*Hourglass, $hr_1 = $hr_2"
			} elseif {$nphases == "2"} {
			  GiD_WriteCalculationFile puts "*Hourglass, $hr_1 = $hr_2 ,$hr_3"
			}
		} elseif {$hr_gls == "No"} {
		}
	}
		if {$hca == "No"} {
		} elseif {$hca == "Yes"} {
			set list_imp [format_xpath {container[@n="_materials_"]/blockdata/container[@n="_HCA_"]/container[@n="_HCA20_"]/blockdata}]
			set imp_list [$root selectNodes $list_imp]
			set impl_list_str ""
			GiD_WriteCalculationFile puts "*Implicit HCA steps"
			foreach implicit_name $imp_list {
				set impl_list [$implicit_name getAttribute "name"]
				if {$impl_list_str eq ""} {
					set impl_list_str $impl_list
				} else {
					append impl_list_str ,$impl_list
				}
			}
			GiD_WriteCalculationFile puts "$impl_list_str"
		}
	    if {$hca == "No"} {
		} elseif {$hca == "Yes"} {
			set list_imp [format_xpath {container[@n="_materials_"]/blockdata/container[@n="_HCA_"]/container[@n="_HCA21_"]/blockdata}]
			set imp_list [$root selectNodes $list_imp]
			set impl_list_str ""
			GiD_WriteCalculationFile puts "*Recording HCA steps"
			foreach implicit_name $imp_list {
				set impl_list [$implicit_name getAttribute "name"]
				if {$impl_list_str eq ""} {
					set impl_list_str $impl_list
				} else {
					append impl_list_str ,$impl_list
				}
			}
			GiD_WriteCalculationFile puts "$impl_list_str"
		}
		if {$hca == "No"} {
		} elseif {$hca == "Yes"} {
			set list_imp [format_xpath {container[@n="_materials_"]/blockdata/container[@n="_HCA_"]/container[@n="_HCA22_"]/blockdata}]
			set imp_list [$root selectNodes $list_imp]
			set impl_list_str ""
			GiD_WriteCalculationFile puts "*Explicit HCA steps"
			foreach implicit_name $imp_list {
				set impl_list [$implicit_name getAttribute "name"]
				if {$impl_list_str eq ""} {
					set impl_list_str $impl_list
				} else {
					append impl_list_str ,$impl_list
				}
			}
			GiD_WriteCalculationFile puts "$impl_list_str"
		}
		set node [$imat selectNodes [format_xpath {container[@n="_HCA_"]/value[@n="_HCA19_"]}]]
		set HCA19 [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*HCA cycle time"
		GiD_WriteCalculationFile puts "$HCA19"
	#
	# Amplitude
	# 
	
	customlib::WriteString ""
	customlib::WriteString "**"
	customlib::WriteString "** Loading amplitude/signal "
	customlib::WriteString "**"
	customlib::WriteString ""
	
	# create a list of amplitudes
	set xp [format_xpath {container[@n="_amplitudes_"]/blockdata}]
	set list [$root selectNodes $xp]
		
		# loop over all Amplitude
		foreach iamp $list {
		  # Amplitude name
		  set ampname [$iamp getAttribute "name"]
		  set node [$iamp selectNodes [format_xpath {value[@n="_amp1_"]}]]
		  set ampli [$node getAttribute "v"]
		  # ramp
		  if {$ampli == "ramp"} {
			set node [$iamp selectNodes [format_xpath {value[@n="_r1_"]}]]
			set p0 [$node getAttribute "v"]
			set node [$iamp selectNodes [format_xpath {value[@n="_r2_"]}]]
			set p1 [$node getAttribute "v"]
			set node [$iamp selectNodes [format_xpath {value[@n="_r3_"]}]]
			set p2 [$node getAttribute "v"]
			set node [$iamp selectNodes [format_xpath {value[@n="_r4_"]}]]
			set p3 [$node getAttribute "v"]
			GiD_WriteCalculationFile puts "*Amplitude, Name=$ampname, Type=$ampli"
			GiD_WriteCalculationFile puts "$p0, $p1, $p2, $p3"
		  } elseif  {$ampli == "rising_sine"} {
			set node [$iamp selectNodes [format_xpath {value[@n="_rs1_"]}]]
			set p0 [$node getAttribute "v"]
			set node [$iamp selectNodes [format_xpath {value[@n="_rs2_"]}]]
			set p1 [$node getAttribute "v"]
			set node [$iamp selectNodes [format_xpath {value[@n="_rs3_"]}]]
			set p2 [$node getAttribute "v"]
			GiD_WriteCalculationFile puts "*Amplitude, Name=$ampname, Type=$ampli"
			GiD_WriteCalculationFile puts "$p0, $p1, $p2"
		  } elseif {$ampli == "rising_cosine"} {
			set node [$iamp selectNodes [format_xpath {value[@n="_rc1_"]}]]
			set p0 [$node getAttribute "v"]
			set node [$iamp selectNodes [format_xpath {value[@n="_rc2_"]}]]
			set p1 [$node getAttribute "v"]
			set node [$iamp selectNodes [format_xpath {value[@n="_rc3_"]}]]
			set p2 [$node getAttribute "v"]
			GiD_WriteCalculationFile puts "*Amplitude, Name=$ampname, Type=$ampli"
			GiD_WriteCalculationFile puts "$p0, $p1, $p2"
		  } elseif {$ampli == "user"} {
			GiD_WriteCalculationFile puts "*Amplitude, Name=$ampname, Type=user"
		  } elseif {$ampli == "tabular"} {
			GiD_WriteCalculationFile puts "*Amplitude, Name=$ampname, Type=tabular"
			set nodes [$iamp selectNodes [format_xpath {value[@n="_tab_"]/function/functionVariable[@n="interpolator_func"]/value[@n="point"]}]]
            foreach node $nodes {
                set pi [$node getAttribute "v"]
                GiD_WriteCalculationFile puts "$pi"
            }
		  } elseif {$ampli == "periodic"} {
			GiD_WriteCalculationFile puts "*Amplitude, Name=$ampname, Type=periodic"
			set node [$iamp selectNodes [format_xpath {value[@n="_p1_"]}]]
			set p0 [$node getAttribute "v"]
			set node [$iamp selectNodes [format_xpath {value[@n="_p2_"]}]]
			set p1 [$node getAttribute "v"]
			set node [$iamp selectNodes [format_xpath {value[@n="_p3_"]}]]
			set p2 [$node getAttribute "v"]
			set node [$iamp selectNodes [format_xpath {value[@n="_p4_"]}]]
			set p3 [$node getAttribute "v"]
			GiD_WriteCalculationFile puts "$p0, $p1, $p2, $p3"
			set nodes [$iamp selectNodes [format_xpath {value[@n="_p5_"]/function/functionVariable[@n="interpolator_func"]/value[@n="point"]}]]
            foreach node $nodes {
                set p4 [$node getAttribute "v"]
                GiD_WriteCalculationFile puts "$p4"
            }
		  }
		}
		
	#
	# Contact/interaction
	# 
		customlib::WriteString ""
		customlib::WriteString "**"
		customlib::WriteString "** Contact/interaction"
		customlib::WriteString "**"
		customlib::WriteString ""
		# create a list of contacts
		set xp [format_xpath {container[@n="_contact_"]}]
		set list [$root selectNodes $xp]
		foreach iconi $list {
		set node [$iconi selectNodes [format_xpath {value[@n="_cont_"]}]]
		set cont [$node getAttribute "v"]
		set xp [format_xpath {container[@n="_contact_"]/blockdata}]
		set list [$root selectNodes $xp]
		# loop over all contacts
		foreach icon $list {
		  # contact name
		  set conname [$icon getAttribute "name"]
		  set node [$icon selectNodes [format_xpath {value[@n="_contact_model_"]}]]
		  set coni [$node getAttribute "v"]
		  set node [$icon selectNodes [format_xpath {value[@n="_sep_"]}]]
		  set sepr [$node getAttribute "v"]
		  set node [$icon selectNodes [format_xpath {value[@n="_type_"]}]]
		  set ctype [$node getAttribute "v"]
		  set node [$icon selectNodes [format_xpath {value[@n="_sf_1_"]}]]
		  set sf_value [$node getAttribute "v"]
		  if {$cont == "Yes"} {
			if {$sepr == "no"} {
				if {$ctype == "...select..."} {
					GiD_WriteCalculationFile puts "*Interaction, Name=$conname, Mechanical=$coni, No Sepration"
				} elseif {$ctype == "Stiffness_factor"} {
					GiD_WriteCalculationFile puts "*Interaction, Name=$conname, Mechanical=$coni, No Sepration, $ctype = $sf_value"
				} else {
				GiD_WriteCalculationFile puts "*Interaction, Name=$conname, Mechanical=$coni, No Sepration, $ctype"
				}
			} elseif {$sepr == "yes"} {
				if {$ctype == "...select..."} {
					GiD_WriteCalculationFile puts "*Interaction, Name=$conname, Mechanical=$coni"
				} elseif {$ctype == "Stiffness_factor"} {
					GiD_WriteCalculationFile puts "*Interaction, Name=$conname, Mechanical=$coni, $ctype = $sf_value"
				} else {
				GiD_WriteCalculationFile puts "*Interaction, Name=$conname, Mechanical=$coni, $ctype"
				}
			}
		  set node [$icon selectNodes [format_xpath {value[@n="_Frc_model_"]}]]
		  set cmodel [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "*Friction, model=$cmodel"
		  if {$cmodel == "MC"} {
			set node [$icon selectNodes [format_xpath {value[@n="_c_MC1_"]}]]
			set p1 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_MC2_"]}]]
			set p2 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_MC3_"]}]]
			set p3 [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "$p1, $p2, $p3"
		  } elseif {$cmodel == "hypoplastic"} {
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp1_"]}]]
			set ph1 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp2_"]}]]
			set ph2 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp3_"]}]]
			set ph3 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp4_"]}]]
			set ph4 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp5_"]}]]
			set ph5 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp6_"]}]]
			set ph6 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp7_"]}]]
			set ph7 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp8_"]}]]
			set ph8 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp9_"]}]]
			set ph9 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp10_"]}]]
			set ph10 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp11_"]}]]
			set ph11 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp12_"]}]]
			set ph12 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp13_"]}]]
			set ph13 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp14_"]}]]
			set ph14 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp15_"]}]]
			set ph15 [$node getAttribute "v"]
			set node [$icon selectNodes [format_xpath {value[@n="_c_hp16_"]}]]
			set ph16 [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "$ph1, $ph2, $ph3, $ph4, $ph5, $ph6, $ph7, $ph8"
		  GiD_WriteCalculationFile puts "$ph9, $ph10, $ph11, $ph12, $ph13, $ph14, $ph15, $ph16"
		  }
		  
		  customlib::WriteString "** Contact pair"
		  set node [$icon selectNodes [format_xpath {value[@n="_Dis_"]}]]
		  set dis [$node getAttribute "v"]
			  
		  set group_names_master {}
		  set group_names_slave {}

		  set xp [format_xpath {condition[@n="_mss_"]/group}]
		  set groups [$icon selectNodes $xp]
		  foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			lappend group_names_master $group_name
		  }
		  set xp [format_xpath {condition[@n="_sss_"]/group}]
		  set groups [$icon selectNodes $xp]
		  foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			lappend group_names_slave $group_name
		  }
		  
		  set length_master [llength $group_names_master]
		  set length_slave [llength $group_names_slave]
			
		  if {$length_master == $length_slave} {
			foreach item1 $group_names_master item2 $group_names_slave {
			  customlib::WriteString "*Contact pair, Interaction=$conname, Discretisation=$dis"
			  GiD_WriteCalculationFile puts  "$item2, $item1"
			}
		  } {
			error [= "The number of master and slave surfaces is not the same for interaction" $conname]
		  }
		  } elseif {$cont == "No"} {}
	    } 
		}
		
	#
	# Initial-conditions
	# 
	customlib::WriteString ""
	customlib::WriteString "**"
	customlib::WriteString "** Initial Conditions"
	customlib::WriteString "**"
	customlib::WriteString ""
	
		set xp [format_xpath {container[@n="_initial_conditions_"]/container[@n="_initial_void_c"]/blockdata[@n="_initial_void_"]/condition[@n="_ini_void_"]}]
		set list [$root selectNodes $xp]
		set wrote_initial_void 0
		if {[llength $list] > 0} {
		  foreach ivoid $list {
		  set xp "./group"
		  set groups [$ivoid selectNodes $xp]
			foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			set node [$group selectNodes [format_xpath {value[@n="_vo_1_"]}]]
			set vr1 [$node getAttribute "v"]
			  if {[string length $vr1] > 0} {
				if {$wrote_initial_void == 0} {
				GiD_WriteCalculationFile puts "*initial conditions, type=void ratio, default"
				set wrote_initial_void 1
				}
				GiD_WriteCalculationFile puts "$group_name, $vr1"
			  }
			}
		  }
		} else {}

		set xp [format_xpath {container[@n="_initial_conditions_"]/container[@n="_initial_stress_c"]/blockdata[@n="_initial_stress_"]/condition[@n="_ini_stress_"]}]
		set list [$root selectNodes $xp]
		set wrote_initial_stress 0
		if {[llength $list] > 0} {
		  foreach istress $list {
		  set xp "./group"
		  set groups [$istress selectNodes $xp]
			foreach group $groups {
			  set group_name [get_domnode_attribute $group n]
			  set node [$group selectNodes [format_xpath {value[@n="_str1_"]}]]
			  set vs1 [$node getAttribute "v"]
			  set node [$group selectNodes [format_xpath {value[@n="_str2_"]}]]
			  set vs2 [$node getAttribute "v"]
			  set node [$group selectNodes [format_xpath {value[@n="_str3_"]}]]
			  set vs3 [$node getAttribute "v"]
			  set node [$group selectNodes [format_xpath {value[@n="_str4_"]}]]
			  set vs4 [$node getAttribute "v"]
			  set node [$group selectNodes [format_xpath {value[@n="_str5_"]}]]
			  set vs5 [$node getAttribute "v"]
			  set node [$group selectNodes [format_xpath {value[@n="_str6_"]}]]
			  set vs6 [$node getAttribute "v"]
			  if {[string length $vs1] > 0} {
			  if {[string length $vs2] > 0} {
				if {$wrote_initial_stress == 0} {
				  GiD_WriteCalculationFile puts "*initial conditions, type=stress, geostatic"
				set wrote_initial_stress 1
				}
				  GiD_WriteCalculationFile puts "$group_name, $vs1, $vs2, $vs3, $vs4, $vs5, $vs6"
			  }
			  }
			}
		  }
		} else {}
		
		set xp [format_xpath {container[@n="_initial_conditions_"]/container[@n="_pore_water_c"]/blockdata[@n="_pore_water_pressure_"]/condition[@n="_po_water_"]}]
		set list [$root selectNodes $xp]
		set wrote_initial_pore_wa 0
		  if {[llength $list] > 0} {
		  foreach powa $list {
		  set xp "./group"
		  set groups [$powa selectNodes $xp]
			foreach group $groups {
			  set group_name [get_domnode_attribute $group n]
			  set node [$group selectNodes [format_xpath {value[@n="_pwp1_"]}]]
			  set pw1 [$node getAttribute "v"]
			  set node [$group selectNodes [format_xpath {value[@n="_pwp2_"]}]]
			  set pw2 [$node getAttribute "v"]
			  set node [$group selectNodes [format_xpath {value[@n="_pwp3_"]}]]
			  set pw3 [$node getAttribute "v"]
			  set node [$group selectNodes [format_xpath {value[@n="_pwp4_"]}]]
			  set pw4 [$node getAttribute "v"]
			  if {[string length $pw1] > 0} {
			  if {[string length $pw2] > 0} {
				if {$wrote_initial_pore_wa == 0} {
				   GiD_WriteCalculationFile puts "*initial conditions, type=pore water pressure, default"
				set wrote_initial_pore_wa 1
				}
				  GiD_WriteCalculationFile puts "$group_name, $pw1, $pw2, $pw3, $pw4"
			  }
			  }
			}
		  }
		  } else {}
	
		set xp [format_xpath {container[@n="_initial_conditions_"]/container[@n="_pore_air_c"]/blockdata[@n="_pore_air_pressure_"]/condition[@n="_po_air_"]}]
		set list [$root selectNodes $xp]
		set wrote_initial_pore_air 0
		  if {[llength $list] > 0} {
			
		  foreach powa $list {
		  set xp "./group"
		  set groups [$powa selectNodes $xp]
			foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			set node [$group selectNodes [format_xpath {value[@n="_pap1_"]}]]
			set pa1 [$node getAttribute "v"]
			set node [$group selectNodes [format_xpath {value[@n="_pap2_"]}]]
			set pa2 [$node getAttribute "v"]
			set node [$group selectNodes [format_xpath {value[@n="_pap3_"]}]]
			set pa3 [$node getAttribute "v"]
			set node [$group selectNodes [format_xpath {value[@n="_pap4_"]}]]
			set pa4 [$node getAttribute "v"]
			  if {[string length $pa1] > 0} {
			  if {[string length $pa2] > 0} {
				if {$wrote_initial_pore_air == 0} {
				   GiD_WriteCalculationFile puts "*initial conditions, type=pore air pressure, default"
				set wrote_initial_pore_air 1
				}
				  GiD_WriteCalculationFile puts "$group_name, $pa1, $pa2, $pa3, $pa4"
			  }
			  }
			}
		  }
		  } else {}
			
		set xp [format_xpath {container[@n="_initial_conditions_"]/container[@n="_state_var_"]/blockdata[@n="_state_variable_"]}]
		set list [$root selectNodes $xp]
		# Initialize flags to track if the lines have been written
		set default_written 0
		set user_written 0
		set fluctuation_written 0
		foreach stvar $list {
		  set xp "./condition\[@n='_sta_var_'\]/group"
		  set groups [$stvar selectNodes $xp]
		  foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			set node [$group selectNodes [format_xpath {value[@n="_stv1_"]}]]
			set stva1 [$node getAttribute "v"]
			set node [$group selectNodes [format_xpath {value[@n="_stv2_"]}]]
			set stva2 [$node getAttribute "v"]
			set node [$group selectNodes [format_xpath {value[@n="_stv3_"]}]]
			set stva3 [$node getAttribute "v"]
			set node [$group selectNodes [format_xpath {value[@n="_stv4_"]}]]
			set stva4 [$node getAttribute "v"]
			set node [$group selectNodes [format_xpath {value[@n="_stv5_"]}]]
			set stva5 [$node getAttribute "v"]
			set node [$group selectNodes [format_xpath {value[@n="_stv6_"]}]]
			set stva6 [$node getAttribute "v"]
			if {[llength $stva2] > 0} {
			  if {$stva1 == "default"} {
				if {!$default_written} {
				  GiD_WriteCalculationFile puts "*initial conditions, type=state variables"
				  set default_written 1
				}
				GiD_WriteCalculationFile puts "$group_name, $stva2, $stva3"
			  } elseif {$stva1 == "user"} {
				if {!$user_written} {
				  GiD_WriteCalculationFile puts "*initial conditions, type=state variables, $stva1"
				  set user_written 1
				}
				GiD_WriteCalculationFile puts "$group_name"
			  } elseif {$stva1 == "fluctuation"} {
				if {!$fluctuation_written} {
				  GiD_WriteCalculationFile puts "*initial conditions, type=state variables, $stva1"
				  set fluctuation_written 1
				}
				GiD_WriteCalculationFile puts "$group_name, $stva4, $stva5, $stva6"
			  }
			}
		  }
		}
		
	#
	# Steps
	# 

	# create a list of steps
	set xp [format_xpath {container[@n="_steps_"]/blockdata}]
	set list [$root selectNodes $xp]
	# loop over all steps
	set step_num 0
	foreach istep $list {
	
	  set step_num [expr $step_num + 1]

      customlib::WriteString ""
	  customlib::WriteString "**"
	  GiD_WriteCalculationFile puts "** Step $step_num"
	  customlib::WriteString "**"
	  customlib::WriteString ""
	
	  # material name and number of phases
	  set stepname [$istep getAttribute "name"]
	  set node [$istep selectNodes [format_xpath {value[@n="_maxiter_"]}]]
	  set maxiter [$node getAttribute "v"]
	  set node [$istep selectNodes [format_xpath {value[@n="_maxinc_"]}]]
	  set maxinc [$node getAttribute "v"]
	  set node [$istep selectNodes [format_xpath {value[@n="_nlgeom_"]}]]
	  set nlgeom [$node getAttribute "v"]
	  if {$maxiter == "default"} {
        if {$nlgeom == "yes"} {
          GiD_WriteCalculationFile puts "*Step, name=$stepname, inc=$maxinc, nlgeom"
        } else {
          GiD_WriteCalculationFile puts "*Step, name=$stepname, inc=$maxinc"
        }
      } else {
        if {$nlgeom == "yes"} {
          GiD_WriteCalculationFile puts "*Step, name=$stepname, maxiter=$maxiter, inc=$maxinc, nlgeom"
        } else {
          GiD_WriteCalculationFile puts "*Step, name=$stepname, maxiter=$maxiter, inc=$maxinc"
        }
      }

	  #
	  # Analysis type: Static, Transient, Dynamic
	  #

	  set node [$istep selectNodes [format_xpath {value[@n="_analysis_"]}]]
	  set analysis [$node getAttribute "v"]
	  GiD_WriteCalculationFile puts "*$analysis"

	  #
	  # Time integration settings
	  #
	  if {$analysis == "Geostatic"} {
	    GiD_WriteCalculationFile puts ""
	  } elseif {$analysis == "Reduction"} {
		  set node [$istep selectNodes [format_xpath {container[@n="_factor_of_saftey_"]/value[@n="_Fos_"]}]]
		  set p5 [$node getAttribute "v"]
		  set node [$istep selectNodes [format_xpath {container[@n="_factor_of_saftey_"]/value[@n="_dFos_"]}]]
		  set p6 [$node getAttribute "v"]
	    GiD_WriteCalculationFile puts "$p5, $p6"
	  } else {
	  set node [$istep selectNodes [format_xpath {container[@n="_time_integration_"]/value[@n="_dt0_"]}]]
	  set p1 [$node getAttribute "v"]
	  set node [$istep selectNodes [format_xpath {container[@n="_time_integration_"]/value[@n="_t_"]}]]
	  set p2 [$node getAttribute "v"]
	  set node [$istep selectNodes [format_xpath {container[@n="_time_integration_"]/value[@n="_dtmin_"]}]]
	  set p3 [$node getAttribute "v"]
	  set node [$istep selectNodes [format_xpath {container[@n="_time_integration_"]/value[@n="_dtmax_"]}]]
	  set p4 [$node getAttribute "v"]
	  
	  GiD_WriteCalculationFile puts "$p1, $p2, $p3, $p4"
	  if {$analysis == "Dynamic"} {
	    set node [$istep selectNodes [format_xpath {container[@n="_time_integration_"]/value[@n="_time_integration_scheme_"]}]]
	    set p1 [$node getAttribute "v"]
		if {$p1 == "Backward Euler"} {
		  GiD_WriteCalculationFile puts "*$p1"
		} elseif {$p1 == "Hilber-Hughes-Taylor"} {
		  set node [$istep selectNodes [format_xpath {container[@n="_time_integration_"]/value[@n="_hht_alpha_"]}]]
	      set p2 [$node getAttribute "v"]
		  GiD_WriteCalculationFile puts "*$p1, alpha=$p2"
		}
	  }
   	  }

	  #
	  # Solver
	  #

	  set node [$istep selectNodes [format_xpath {container[@n="_solution_settings_"]/value[@n="_solver_"]}]]
	  set solver [$node getAttribute "v"]
	  GiD_WriteCalculationFile puts "*Solver, $solver"

      #
	  # Extrapolation
	  #

	  set node [$istep selectNodes [format_xpath {container[@n="_extrapolations_"]/value[@n="_extrapolation_"]}]]
	  set extra_p [$node getAttribute "v"]
	  set node [$istep selectNodes [format_xpath {container[@n="_extrapolations_"]/value[@n="_extrap_"]}]]
	  set extrap_ [$node getAttribute "v"]
	  if {$extra_p == "Yes"} {
		  GiD_WriteCalculationFile puts "*Extrapolation, $extrap_"
	  } else {
	  }
	  
	  #
	  # Reset
	  #
	  set node [$istep selectNodes [format_xpath {container[@n="_reset_"]/value[@n="_rest_"]}]]
	  set re0 [$node getAttribute "v"]
	  set node [$istep selectNodes [format_xpath {container[@n="_reset_"]/value[@n="_rest1_"]}]]
	  set re1 [get_domnode_attribute $node v] 
	  set node [$istep selectNodes [format_xpath {container[@n="_reset_"]/value[@n="_rest2_"]}]]
	  set re2 [get_domnode_attribute $node v] 
	  set node [$istep selectNodes [format_xpath {container[@n="_reset_"]/value[@n="_rest3_"]}]]
	  set re3 [get_domnode_attribute $node v]
	  set node [$istep selectNodes [format_xpath {container[@n="_reset_"]/value[@n="_rest4_"]}]]
	  set re4 [get_domnode_attribute $node v]
        if {$re0=="Yes"} {
			GiD_WriteCalculationFile puts "*Reset"
				if {$re1 == "Yes"} {append _reset_list_ "Displacement, "}
				if {$re2 == "Yes"} {append _reset_list_ "Strain, "}
				if {$re3 == "Yes"} {append _reset_list_ "Stress, "}
				if {$re4 == "Yes"} {append _reset_list_ "Void ratio, "}
				# Write calculation file if there are terms to output
				if {[string length $_reset_list_] > 0} {
				GiD_WriteCalculationFile puts "$_reset_list_"
			}
			unset _reset_list_
		} else {
		}

	  #
	  # Boundary conditions applied to the solid phase
	  #
	  
	  set xp "./container\[@n='_boundary_conditions_'\]/condition\[@n='_solid_'\]/group"
	  set groups [$istep selectNodes $xp]
      foreach group $groups {
	    set group_name [get_domnode_attribute $group n]
	    set node [$group selectNodes "./value\[@n = '_type_'\]"]
	    set bc_type [get_domnode_attribute $node v]
        if {$bc_type == "displacement"} {
	      GiD_WriteCalculationFile puts "*Boundary"
		} else {
		  GiD_WriteCalculationFile puts "*Boundary, $bc_type"
		}
		set node [$group selectNodes "./value\[@n = '_x_'\]"]
	    set bc_magnitudex [get_domnode_attribute $node v]
			if {$bc_magnitudex == "free"} {
			} elseif {$bc_magnitudex == "fixed"} {
			  GiD_WriteCalculationFile puts "$group_name, u1, 0"
			} else {
			set node [$group selectNodes "./value\[@n = '_xp_'\]"]
			set bc_magnitudexp [get_domnode_attribute $node v]
			GiD_WriteCalculationFile puts "$group_name, u1, $bc_magnitudexp"
			}
		  set node [$group selectNodes "./value\[@n = '_y_'\]"]
	      set bc_magnitudey [get_domnode_attribute $node v]
			if {$bc_magnitudey == "free"} {
			} elseif {$bc_magnitudey == "fixed"} {
			  GiD_WriteCalculationFile puts "$group_name, u2, 0"
			} else {
			set node [$group selectNodes "./value\[@n = '_yp_'\]"]
			set bc_magnitudeyp [get_domnode_attribute $node v]
			GiD_WriteCalculationFile puts "$group_name, u2, $bc_magnitudeyp"
			}
			GiD_WriteCalculationFile puts ""
		  set node [$group selectNodes "./value\[@n = '_z_'\]"]
	      set bc_magnitudez [get_domnode_attribute $node v]
			if {$bc_magnitudez == "free"} {
			GiD_WriteCalculationFile puts ""
			} elseif {$bc_magnitudez == "fixed"} {
			  GiD_WriteCalculationFile puts "$group_name, u3, 0"
			} else {
			set node [$group selectNodes "./value\[@n = '_zp_'\]"]
			set bc_magnitudezp [get_domnode_attribute $node v]
			GiD_WriteCalculationFile puts "$group_name, u3, $bc_magnitudezp"
			}
	  }
	  
	  #
	  # Boundary conditions applied to the pore fuid (1) phase (pore water)
	  #
	  
	  set xp "./container\[@n='_boundary_conditions_'\]/condition\[@n='_fluid1_'\]/group"
	  set groups [$istep selectNodes $xp]
      foreach group $groups {
	    set group_name [get_domnode_attribute $group n]
	    GiD_WriteCalculationFile puts "*Boundary, type=hydrostatic"
		set node [$group selectNodes "./value\[@n = '_vaue_'\]"]
	    set bc_magnitudepw [get_domnode_attribute $node v]
		set node [$group selectNodes "./value\[@n = '_zero_'\]"]
	    set zero [get_domnode_attribute $node v]
		GiD_WriteCalculationFile puts "$group_name, pw, $bc_magnitudepw, $zero"
	  }  
	  
	  #
	  # Constraints - Equal Degree of Freedom (EqualDOF)
	  #
	  
	  set xp "./container\[@n='_constraints_'\]/condition\[@n='_equal_dof_'\]/groupList"
	  set groupLists [$istep selectNodes $xp]
      foreach groupList $groupLists {
	    set xp2 "./group"
	    set groups [$groupLists selectNodes $xp2]
		set names {}
		foreach group $groups {
		  set name [get_domnode_attribute $group n]
		  lappend names $name
		}
	    set master_set [lindex $names 0]
		set slave_set [lindex $names 1]
	    GiD_WriteCalculationFile puts "*Equation, type=Equal DOF"
		set node [$groupList selectNodes "./value\[@n = '_dof_'\]"]
	    set dof [get_domnode_attribute $node v]
		set node [$groupList selectNodes "./value\[@n = '_penalty_'\]"]
	    set penalty [get_domnode_attribute $node v]
		GiD_WriteCalculationFile puts "*Penalty = $penalty"
		GiD_WriteCalculationFile puts "$master_set, $slave_set, $dof"
	  }  
	  
	  #
	  # Save reaction
	  #
	  set xp "./container\[@n='_Rec_for_'\]/condition\[@n='_reaction_forces_'\]/group"
	  set groups [$istep selectNodes $xp]
      foreach group $groups {
	    set group_name [get_domnode_attribute $group n]
		set node [$group selectNodes "./value\[@n = '_x_di_'\]"]
		set x_di [get_domnode_attribute $node v]
		if {$x_di == "Yes"} {
		  GiD_WriteCalculationFile puts "*SAVE Reaction force"
		  GiD_WriteCalculationFile puts "$group_name, 1" 
		} else {
			GiD_WriteCalculationFile puts ""
		}
		set node [$group selectNodes "./value\[@n = '_y_di_'\]"]
		set y_di [get_domnode_attribute $node v]
		if {$y_di == "Yes"} {
		  GiD_WriteCalculationFile puts "*SAVE Reaction force"
		  GiD_WriteCalculationFile puts "$group_name, 2" 
		} else {
			GiD_WriteCalculationFile puts ""
		}
		set node [$group selectNodes "./value\[@n = '_z_di_'\]"]
		set z_di [get_domnode_attribute $node v]
		if {$z_di == "Yes"} {
		  GiD_WriteCalculationFile puts "*SAVE Reaction force"
		  GiD_WriteCalculationFile puts "$group_name, 3" 
		} else {
			GiD_WriteCalculationFile puts ""
		}
	  }
	
	  #
	  # Save reaction
	  #
	    set xp "./container\[@n='_Rec_for_'\]/condition\[@n='_USRfo_'\]/group"
	    set groups [$istep selectNodes $xp]
        foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			set node [$group selectNodes "./value\[@n = '_USRx_di_'\]"]
			set Ux_di [get_domnode_attribute $node v]
			if {$Ux_di == "Yes"} {
				set node [$group selectNodes "./value\[@n = '_loading_rate1_'\]"]
				set lr_1 [get_domnode_attribute $node v]
				if {$lr_1 == "instant"} {
				GiD_WriteCalculationFile puts "*Reaction force, instant"
				GiD_WriteCalculationFile puts "$group_name, 1" 
				} elseif {$lr_1 == "amplitude"} {
				set node [$group selectNodes "./value\[@n = '_amplitude1_'\]"]
				set amplname [get_domnode_attribute $node v]
				GiD_WriteCalculationFile puts "*Reaction force, amplitude=$amplname"
				GiD_WriteCalculationFile puts "$group_name, 1" 
				}
			}
			set node [$group selectNodes "./value\[@n = '_USRy_di_'\]"]
			set Uy_di [get_domnode_attribute $node v]
			if {$Uy_di == "Yes"} {
				set node [$group selectNodes "./value\[@n = '_loading_rate2_'\]"]
				set lr_2 [get_domnode_attribute $node v]
				if {$lr_2 == "instant"} {
				GiD_WriteCalculationFile puts "*Reaction force, instant"
				GiD_WriteCalculationFile puts "$group_name, 2" 
				} elseif {$lr_2 == "amplitude"} {
				set node [$group selectNodes "./value\[@n = '_amplitude1_'\]"]
				set amplname [get_domnode_attribute $node v]
				GiD_WriteCalculationFile puts "*Reaction force, amplitude=$amplname"
				GiD_WriteCalculationFile puts "$group_name, 2" 
				}
			}
			set node [$group selectNodes "./value\[@n = '_USRz_di_'\]"]
			set Uz_di [get_domnode_attribute $node v]
			if {$Uz_di == "Yes"} {
			  set node [$group selectNodes "./value\[@n = '_loading_rate3_'\]"]
				set lr_3 [get_domnode_attribute $node v]
				if {$lr_3 == "instant"} {
				GiD_WriteCalculationFile puts "*Reaction force, instant"
				GiD_WriteCalculationFile puts "$group_name, 3" 
				} elseif {$lr_3 == "amplitude"} {
				set node [$group selectNodes "./value\[@n = '_amplitude3_'\]"]
				set amplname [get_domnode_attribute $node v]
				GiD_WriteCalculationFile puts "*Reaction force, amplitude=$amplname"
				GiD_WriteCalculationFile puts "$group_name, 3" 
				}
			}
		}
		
	  #
	  # User Boundary conditions
	  #
	  set xp "./container\[@n='_boundary_conditions_'\]/condition\[@n='_Uboundary_'\]/group"
	  set groups [$istep selectNodes $xp]
      foreach group $groups {
	    set group_name [get_domnode_attribute $group n]
		set node [$group selectNodes "./value\[@n = '_sl_pw_'\]"]
	    set Ubound [get_domnode_attribute $node v]
		set node [$group selectNodes "./value\[@n = '_val_'\]"]
	    set val [get_domnode_attribute $node v]
			if {$Ubound == "...select.."} {
			} elseif {$Ubound == "Displacement in x-direction"} {
			GiD_WriteCalculationFile puts "*Uboundary"
			GiD_WriteCalculationFile puts "$group_name, u1, $val"
			} elseif {$Ubound == "Displacement in y-direction"} {
			GiD_WriteCalculationFile puts "*Uboundary"
			GiD_WriteCalculationFile puts "$group_name, u2, $val"
			} elseif {$Ubound == "Displacement in z-direction"} {
			GiD_WriteCalculationFile puts "*Uboundary"
			GiD_WriteCalculationFile puts "$group_name, u3, $val"
			} elseif {$Ubound == "Pore-water pressure"} {
			GiD_WriteCalculationFile puts "*Uboundary"
			GiD_WriteCalculationFile puts "$group_name, pw, $val"
			} elseif {$Ubound == "Pore-air pressure"} {
			GiD_WriteCalculationFile puts "*Uboundary"
			GiD_WriteCalculationFile puts "$group_name, pa, $val"
			} elseif {$Ubound == "Water displacement in x-direction"} {
			GiD_WriteCalculationFile puts "*Uboundary"
			GiD_WriteCalculationFile puts "$group_name, w1, $val"
			} elseif {$Ubound == "Water displacement in y-direction"} {
			GiD_WriteCalculationFile puts "*Uboundary"
			GiD_WriteCalculationFile puts "$group_name, w2, $val"
			} elseif {$Ubound == "Water displacement in z-direction"} {
			GiD_WriteCalculationFile puts "*Uboundary"
			GiD_WriteCalculationFile puts "$group_name, w3, $val"
			} 
		}

	  #
	  # Loads (von Neumann BCs)
	  #
	  
	  # Drainage
	  set xp "./container\[@n='_loads_'\]/condition\[@n='_drain_'\]/group"
	  set groups [$istep selectNodes $xp]
      foreach group $groups {
	    set group_name [get_domnode_attribute $group n]
		set node [$group selectNodes "./value\[@n = '_drainage_'\]"]
	    set drain_mag [get_domnode_attribute $node v]
			GiD_WriteCalculationFile puts "*DSload, instant"
			GiD_WriteCalculationFile puts "$group_name, drainage-w, $drain_mag"
		}
	  
	  # Gravitational forces
	  set xp "./container\[@n='_loads_'\]/condition\[@n='_gravity_'\]/group"
	  set groups [$istep selectNodes $xp]
      foreach group $groups {
	    set group_name [get_domnode_attribute $group n]
		set node [$group selectNodes "./value\[@n = '_loading_rate_'\]"]
	    set loading_rate [get_domnode_attribute $node v]
		set node [$group selectNodes "./value\[@n = '_value_'\]"]
	    set magnitude [get_domnode_attribute $node v]
		set node [$group selectNodes "./value\[@n = '_direction_'\]"]
		set direction [get_domnode_attribute $node v]
		if {$loading_rate == "instant"} {
		  GiD_WriteCalculationFile puts "*Body force, instant"
		  GiD_WriteCalculationFile puts "$group_name, Grav, $magnitude, $direction"
		} elseif {$loading_rate == "amplitude"} {
		  set node [$group selectNodes "./value\[@n = '_amplitude_'\]"]
	      set amplname [get_domnode_attribute $node v]
		  GiD_WriteCalculationFile puts "*Body force, amplitude=$amplname"
		  GiD_WriteCalculationFile puts "$group_name, Grav, $magnitude, $direction"
	    }
	  }
	  
	  # Distributed load acting normal to the line
	  set xp "./container\[@n='_loads_'\]/condition\[@n='_dis_load_'\]/group"
	  set groups [$istep selectNodes $xp]
      foreach group $groups {
	    set group_name [get_domnode_attribute $group n]
		set node [$group selectNodes "./value\[@n = '_d_loading_rate_'\]"]
	    set loading_rate [get_domnode_attribute $node v]
		if {$loading_rate == "instant"} {
		  GiD_WriteCalculationFile puts "*DLoad, instant"
		} elseif {$loading_rate == "amplitude"} {
		  set node [$group selectNodes "./value\[@n = '_amplitude_'\]"]
	      set amplname [get_domnode_attribute $node v]
		  GiD_WriteCalculationFile puts "*DLoad, amplitude=$amplname"
	    } 
		set node [$group selectNodes "./value\[@n = '_Load_label_'\]"]
	    set loadl [get_domnode_attribute $node v]
		set node [$group selectNodes "./value\[@n = '_d_magnitude_'\]"]
	    set magnitude [get_domnode_attribute $node v]
		if {$magnitude != 0.0} {
		  GiD_WriteCalculationFile puts "$group_name, $loadl, $magnitude"
		}
	  }
	  
	  # pressure acting normal to the surface/line
	  set xp "./container\[@n='_loads_'\]/condition\[@n='_surface_pressure_'\]/group"
	  set groups [$istep selectNodes $xp]
      foreach group $groups {
	    set group_name [get_domnode_attribute $group n]
		set node [$group selectNodes "./value\[@n = '_loading_rate_'\]"]
	    set loading_rate [get_domnode_attribute $node v]
		if {$loading_rate == "instant"} {
		  GiD_WriteCalculationFile puts "*DSload, instant"
		} elseif {$loading_rate == "amplitude"} {
		  set node [$group selectNodes "./value\[@n = '_amplitude_'\]"]
	      set amplname [get_domnode_attribute $node v]
		  GiD_WriteCalculationFile puts "*DSload, amplitude=$amplname"
	    }
		set node [$group selectNodes "./value\[@n = '_value_'\]"]
	    set magnitude [get_domnode_attribute $node v]
		GiD_WriteCalculationFile puts "$group_name, P, $magnitude"
	  }
	  
	  # concentrated load (point load)
	  set xp "./container\[@n='_loads_'\]/condition\[@n='_concentrated_load_'\]/group"
	  set groups [$istep selectNodes $xp]
      foreach group $groups {
	    set group_name [get_domnode_attribute $group n]
		set node [$group selectNodes "./value\[@n = '_loading_rate_'\]"]
	    set loading_rate [get_domnode_attribute $node v]
		if {$loading_rate == "instant"} {
		  GiD_WriteCalculationFile puts "*CLoad, instant"
		} elseif {$loading_rate == "amplitude"} {
		  set node [$group selectNodes "./value\[@n = '_amplitude_'\]"]
	      set amplname [get_domnode_attribute $node v]
		  GiD_WriteCalculationFile puts "*CLoad, amplitude=$amplname"
	    } 
		set node [$group selectNodes "./value\[@n = '_value_x_'\]"]
	    set magnitude [get_domnode_attribute $node v]
		if {$magnitude != 0.0} {
		  GiD_WriteCalculationFile puts "$group_name, 1, $magnitude"
		}
		set node [$group selectNodes "./value\[@n = '_value_y_'\]"]
	    set magnitude [get_domnode_attribute $node v]
		if {$magnitude != 0.0} {
		  GiD_WriteCalculationFile puts "$group_name, 2, $magnitude"
		}
		set node [$group selectNodes "./value\[@n = '_value_z_'\]"]
	    set magnitude [get_domnode_attribute $node v]
		if {$magnitude != 0.0} {
		  GiD_WriteCalculationFile puts "$group_name, 3, $magnitude"
		}
	  }

	  # Compliant base
	  set xp "./container\[@n='_loads_'\]/condition\[@n='_compliant_base_'\]/group"
	  set groups [$istep selectNodes $xp]
      foreach group $groups {
	    set group_name [get_domnode_attribute $group n]
		set node [$group selectNodes "./value\[@n = '_density_'\]"]
		set density [get_domnode_attribute $node v]
		set node [$group selectNodes "./value\[@n = '_wave_vel_'\]"]
	    set wave_vel [get_domnode_attribute $node v]
		set node [$group selectNodes "./value\[@n = '_direction_'\]"]
	    set dir [get_domnode_attribute $node v]
		set node [$group selectNodes "./value\[@n = '_amplitude_'\]"]
	    set ampl [get_domnode_attribute $node v]
		set factor [expr $density * $wave_vel]
		GiD_WriteCalculationFile puts "*DSload, amplitude=$ampl"
		GiD_WriteCalculationFile puts "$group_name, compliant, $dir, $factor"
	  }
		
	  #
	  # Visual Output
	  #
	  
	  set node [$istep selectNodes [format_xpath {container[@n="_output_"]/container[@n="_outputs_"]/value[@n="_feq_"]}]]
	  set feq [$node getAttribute "v"]
		GiD_WriteCalculationFile puts "*output, field, vtk, ascii"
		GiD_WriteCalculationFile puts "*frequency=$feq"
		
	  # Node output
	  set xp "./container\[@n='_output_'\]/container\[@n='_outputs_'\]/condition\[@n='_node_output_'\]/group"
	  set groups [$istep selectNodes $xp]
        foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			set node [$group selectNodes "./value\[@n = '_solid_dis_'\]"]
			set sol_dis [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_solid_vel_'\]"]
			set sol_vel [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_solid_acc_'\]"]
			set sol_acc [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_po_wa_pr_'\]"]
			set po_wa [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_water_disp_'\]"]
			set wa_dis [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_po_ai_pr_'\]"]
			set po_ai [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_rec_for_'\]"]
			set r_f [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_stress_'\]"]
			set stress [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_strain_'\]"]
			set strain [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_sets_'\]"]
			set sets [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_vis_stressij_'\]"]
			set mec_vis [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_voi_'\]"]
			set voi [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_sat_eff_'\]"]
			set sat_eff [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_sat_'\]"]
			set sat [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_phantom_elasticity_'\]"]
			set ph_el [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_con_f_'\]"]
			set con_f [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_contact_'\]"]
			set con [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_contact_sdv_'\]"]
			set con_sdv [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_contact_diagnostic_'\]"]
			set con_dia [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_extra_'\]"]
			set ext [get_domnode_attribute $node v]
				GiD_WriteCalculationFile puts "*node output, nset = $group_name"
				# Check each attribute and add it to the output_terms if it's 'Yes'
				if {$sol_dis == "Yes"} {append output_terms "u, "}
				if {$sol_vel == "Yes"} {append output_terms "v, "}
				if {$sol_acc == "Yes"} {append output_terms "a, "}
				if {$po_wa == "Yes"} {append output_terms "pw, "}
				if {$wa_dis == "Yes"} {append output_terms "uw, "}
				if {$po_ai == "Yes"} {append output_terms "pa, "}
				if {$r_f == "Yes"} {append output_terms "rf, "}
				if {$stress == "Yes"} {append output_terms "S, "}
				if {$strain == "Yes"} {append output_terms "e, "}
				if {$sets == "Yes"} {append output_terms "sets, "}
				if {$voi == "Yes"} {append output_terms "void, "}
				if {$mec_vis == "Yes"} {append output_terms "vis_stressij, "}
				if {$sat_eff == "Yes"} {append output_terms "Sat_eff, "}
				if {$sat == "Yes"} {append output_terms "sat, "}
				if {$ph_el == "Yes"} {append output_terms "Phantom_Elasticity, "}
				if {$con_f == "Yes"} {append output_terms "Con_f, "}
				if {$con == "Yes"} {append output_terms "Contact, "}
				if {$con_sdv == "Yes"} {append output_terms "Contact_sdv, "}
				if {$con_dia == "Yes"} {append output_terms "Contact_diagnostic, "}
				# Write calculation file if there are terms to output
				if {[string length $output_terms] > 0} {
				GiD_WriteCalculationFile puts "$output_terms $ext"
			}
			unset output_terms
		}
		set xp "./container\[@n='_output_'\]/container\[@n='_outputs_'\]/condition\[@n='_element_output_'\]/group"
	    set groups [$istep selectNodes $xp]
        foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			set node [$group selectNodes "./value\[@n = '_e_stress_'\]"]
			set e_stress [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_strain_'\]"]
			set e_strain [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_vis_stressij_'\]"]
			set e_mec_vis [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_void_'\]"]
			set e_voi [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_sat_eff_'\]"]
			set e_sat_eff [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_sat_'\]"]
			set e_sat [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_phantom_elasticity_'\]"]
			set e_ph_el [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_sets_'\]"]
			set e_set [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_mat_'\]"]
			set e_mat [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_contact_'\]"]
			set e_con [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_contact_sdv_'\]"]
			set e_con_sdv [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_contact_diagnostic_'\]"]
			set e_con_dia [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_extra_'\]"]
			set e_ext [get_domnode_attribute $node v]
				GiD_WriteCalculationFile puts "*Element output, elset = $group_name"
				# Check each attribute and add it to the e_output_terms if it's 'Yes'
				if {$e_stress == "Yes"} {append e_output_terms "S, "}
				if {$e_strain == "Yes"} {append e_output_terms "e, "}
				if {$e_set == "Yes"} {append e_output_terms "sets, "}
				if {$e_voi == "Yes"} {append e_output_terms "void, "}
				if {$e_mec_vis == "Yes"} {append e_output_terms "vis_stressij, "}
				if {$e_sat_eff == "Yes"} {append e_output_terms "Sat_eff, "}
				if {$e_sat == "Yes"} {append e_output_terms "sat, "}
				if {$e_ph_el == "Yes"} {append e_output_terms "Phantom_Elasticity, "}
				if {$e_mat == "Yes"} {append e_output_terms "mat, "}
				if {$e_con == "Yes"} {append e_output_terms "Contact, "}
				if {$e_con_sdv == "Yes"} {append e_output_terms "Contact_sdv, "}
				if {$e_con_dia == "Yes"} {append e_output_terms "Contact_diagnostic, "}
				# Write calculation file if there are terms to output
				if {[string length $e_output_terms] > 0} {
				GiD_WriteCalculationFile puts "$e_output_terms $e_ext"
			}
			unset e_output_terms
		}
		
	  #
	  # Print Output
	  #
		
	  # Node output
	  set xp "./container\[@n='_output_'\]/container\[@n='_outputs_print_'\]/condition\[@n='_node_output_p_'\]/group"
	  set groups [$istep selectNodes $xp]
	  GiD_WriteCalculationFile puts "*Output, print"
        foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			set node [$group selectNodes "./value\[@n = '_solid_dis_p_'\]"]
			set sol_dis [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_solid_vel_p_'\]"]
			set sol_vel [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_solid_acc_p_'\]"]
			set sol_acc [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_po_wa_pr_p_'\]"]
			set po_wa [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_water_disp_p_'\]"]
			set wa_dis [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_po_ai_pr_p_'\]"]
			set po_ai [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_rec_for_p_'\]"]
			set r_f [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_stress_p_'\]"]
			set stress [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_strain_p_'\]"]
			set strain [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_sets_p_'\]"]
			set sets [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_vis_stressij_p_'\]"]
			set mec_vis [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_voi_p_'\]"]
			set voi [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_sat_eff_p_'\]"]
			set sat_eff [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_sat_p_'\]"]
			set sat [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_phantom_elasticity_p_'\]"]
			set ph_el [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_con_f_p_'\]"]
			set con_f [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_contact_p_'\]"]
			set con [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_contact_sdv_p_'\]"]
			set con_sdv [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_contact_diagnostic_p_'\]"]
			set con_dia [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_extra_p_'\]"]
			set ext_p [get_domnode_attribute $node v]
				GiD_WriteCalculationFile puts "*node output, nset = $group_name"
				# Check each attribute and add it to the output_terms_p if it's 'Yes'
				if {$sol_dis == "Yes"} {append output_terms_p "u, "}
				if {$sol_vel == "Yes"} {append output_terms_p "v, "}
				if {$sol_acc == "Yes"} {append output_terms_p "a, "}
				if {$po_wa == "Yes"} {append output_terms_p "pw, "}
				if {$wa_dis == "Yes"} {append output_terms_p "uw, "}
				if {$po_ai == "Yes"} {append output_terms_p "pa, "}
				if {$r_f == "Yes"} {append output_terms_p "rf, "}
				if {$stress == "Yes"} {append output_terms_p "S, "}
				if {$strain == "Yes"} {append output_terms_p "e, "}
				if {$sets == "Yes"} {append output_terms_p "sets, "}
				if {$voi == "Yes"} {append output_terms_p "void, "}
				if {$mec_vis == "Yes"} {append output_terms_p "vis_stressij, "}
				if {$sat_eff == "Yes"} {append output_terms_p "Sat_eff, "}
				if {$sat == "Yes"} {append output_terms_p "sat, "}
				if {$ph_el == "Yes"} {append output_terms_p "Phantom_Elasticity, "}
				if {$con_f == "Yes"} {append output_terms_p "Con_f, "}
				if {$con == "Yes"} {append output_terms_p "Contact, "}
				if {$con_sdv == "Yes"} {append output_terms_p "Contact_sdv, "}
				if {$con_dia == "Yes"} {append output_terms_p "Contact_diagnostic, "}
				# Write calculation file if there are terms to output
				if {[string length $output_terms_p] > 0} {
				GiD_WriteCalculationFile puts "$output_terms_p $ext_p"
			}
			unset output_terms_p
		}
		set xp "./container\[@n='_output_'\]/container\[@n='_outputs_print_'\]/condition\[@n='_element_output_p_'\]/group"
	    set groups [$istep selectNodes $xp]
        foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			set node [$group selectNodes "./value\[@n = '_e_stress_p_'\]"]
			set e_stress [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_strain_p_'\]"]
			set e_strain [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_vis_stressij_p_'\]"]
			set e_mec_vis [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_void_p_'\]"]
			set e_voi [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_sat_eff_p_'\]"]
			set e_sat_eff [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_sat_p_'\]"]
			set e_sat [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_phantom_elasticity_p_'\]"]
			set e_ph_el [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_sets_p_'\]"]
			set e_set [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_mat_p_'\]"]
			set e_mat [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_contact_p_'\]"]
			set e_con [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_contact_sdv_p_'\]"]
			set e_con_sdv [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_contact_diagnostic_p_'\]"]
			set e_con_dia [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_e_extra_p_'\]"]
			set e_ext_p [get_domnode_attribute $node v]
				GiD_WriteCalculationFile puts "*Element output, elset = $group_name"
				# Check each attribute and add it to the e_output_terms_p if it's 'Yes'
				if {$e_stress == "Yes"} {append e_output_terms_p "S, "}
				if {$e_strain == "Yes"} {append e_output_terms_p "e, "}
				if {$e_set == "Yes"} {append e_output_terms_p "sets, "}
				if {$e_voi == "Yes"} {append e_output_terms_p "void, "}
				if {$e_mec_vis == "Yes"} {append e_output_terms_p "vis_stressij, "}
				if {$e_sat_eff == "Yes"} {append e_output_terms_p "Sat_eff, "}
				if {$e_sat == "Yes"} {append e_output_terms_p "sat, "}
				if {$e_ph_el == "Yes"} {append e_output_terms_p "Phantom_Elasticity, "}
				if {$e_mat == "Yes"} {append e_output_terms_p "mat, "}
				if {$e_con == "Yes"} {append e_output_terms_p "Contact, "}
				if {$e_con_sdv == "Yes"} {append e_output_terms_p "Contact_sdv, "}
				if {$e_con_dia == "Yes"} {append e_output_terms_p "Contact_diagnostic, "}
				# Write calculation file if there are terms to output
				if {[string length $e_output_terms_p] > 0} {
				GiD_WriteCalculationFile puts "$e_output_terms_p $e_ext_p"
			}
			unset e_output_terms_p
		}
		set xp "./container\[@n='_output_'\]/container\[@n='_outputs_print_'\]/condition\[@n='_i_element_output_p_'\]/group"
	    set groups [$istep selectNodes $xp]
        foreach group $groups {
			set group_name [get_domnode_attribute $group n]
			set node [$group selectNodes "./value\[@n = '_i_e_stress_p_'\]"]
			set e_stress [get_domnode_attribute $node v] 
			set node [$group selectNodes "./value\[@n = '_i_e_strain_p_'\]"]
			set e_strain [get_domnode_attribute $node v] 
			set node [$group selectNodes "./value\[@n = '_i_e_vis_stressij_p_'\]"]
			set e_mec_vis [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_i_e_void_p_'\]"]
			set e_voi [get_domnode_attribute $node v]    
			set node [$group selectNodes "./value\[@n = '_i_e_sat_eff_p_'\]"]
			set e_sat_eff [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_i_e_sat_p_'\]"]
			set e_sat [get_domnode_attribute $node v]    
			set node [$group selectNodes "./value\[@n = '_i_e_phantom_elasticity_p_'\]"]
			set e_ph_el [get_domnode_attribute $node v]  
			set node [$group selectNodes "./value\[@n = '_i_e_sets_p_'\]"]
			set e_set [get_domnode_attribute $node v]    
			set node [$group selectNodes "./value\[@n = '_i_e_mat_p_'\]"]
			set e_mat [get_domnode_attribute $node v]    
			set node [$group selectNodes "./value\[@n = '_i_e_contact_p_'\]"]
			set e_con [get_domnode_attribute $node v]    
			set node [$group selectNodes "./value\[@n = '_i_e_contact_sdv_p_'\]"]
			set e_con_sdv [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_i_e_contact_diagnostic_p_'\]"]
			set e_con_dia [get_domnode_attribute $node v]
			set node [$group selectNodes "./value\[@n = '_i_e_extra_'\]"]
			set i_e_ext [get_domnode_attribute $node v]
				GiD_WriteCalculationFile puts "*Integration point, elset = $group_name"
				# Check each attribute and add it to the _i_e_output_terms_p if it's 'Yes'
				if {$e_stress == "Yes"} {append _i_e_output_terms_p "S, "}
				if {$e_strain == "Yes"} {append _i_e_output_terms_p "e, "}
				if {$e_set == "Yes"} {append _i_e_output_terms_p "sets, "}
				if {$e_voi == "Yes"} {append _i_e_output_terms_p "void, "}
				if {$e_mec_vis == "Yes"} {append _i_e_output_terms_p "vis_stressij, "}
				if {$e_sat_eff == "Yes"} {append _i_e_output_terms_p "Sat_eff, "}
				if {$e_sat == "Yes"} {append _i_e_output_terms_p "sat, "}
				if {$e_ph_el == "Yes"} {append _i_e_output_terms_p "Phantom_Elasticity, "}
				if {$e_mat == "Yes"} {append _i_e_output_terms_p "mat, "}
				if {$e_con == "Yes"} {append _i_e_output_terms_p "Contact, "}
				if {$e_con_sdv == "Yes"} {append _i_e_output_terms_p "Contact_sdv, "}
				if {$e_con_dia == "Yes"} {append _i_e_output_terms_p "Contact_diagnostic, "}
				# Write calculation file if there are terms to output
				if {[string length $_i_e_output_terms_p] > 0} {
				GiD_WriteCalculationFile puts "$_i_e_output_terms_p $i_e_ext"
			}
			unset _i_e_output_terms_p
		}
		GiD_WriteCalculationFile puts "*End Step"
	}
    customlib::WriteString ""
	customlib::WriteString "**"
	customlib::WriteString "** End"
	customlib::WriteString "**"
	customlib::WriteString ""
	GiD_WriteCalculationFile puts "*End Input"
    customlib::EndWriteFile ;#finish writting
}
